import itertools
import numpy as np
import sys
import math
import time
from numpy import linalg as LA
import progressbar
p = progressbar.ProgressBar()

import matplotlib.pyplot as plt

from util import *
from Portfolio import *
from American_Option import *
from Optimal_Stopping import *
from cluster_Prob_Dist_Test import *

np.set_printoptions(precision=3,suppress=True)
#################################################
# Hyperparameters
e = 0.1 # e greedy policy
H = 20000 # Horizon
trails_num = 1
episode_num = 4000

IfPrint = True # True, False

environment = "Portfolio" # Portfolio, American_Option, Optimal_Stopping
#################################################

if environment == "Portfolio":
    env = Portfolio()
    policy = env.ep_binary_policy # policy(theta,s)
    alpha = 0.01 # learning rate
    beta = 0.01 # learning rate

if environment == "American_Option":
    env = American_Option()
    policy = env.binary_policy # policy(theta,s)
    alpha = 0.0025 # learning rate
    beta = 0.0025 # learning rate

if environment == "Optimal_Stopping":
    env = Optimal_Stopping()
    policy = env.binary_policy # policy(theta,s)
    alpha = 0.001 # learning rate
    beta = 0.001 # learning rate


def choose_action(act_dist):
    action_choices = len(act_dist)
    action = np.random.choice(action_choices,p=act_dist)
    return action


def Portfolio_main(i):
    total_r = []
    s = env.reset()
    feature = env.phi(s)
    # theta = feature * 0
    theta = np.random.randn(len(feature))
    Jhat = 25
    Vhat = 10
    # p.start(episode_num)
    for episode in xrange(episode_num):
        # p.update(episode+1)
        # if episode % 10 == 0:
        if episode % 10 == 0 and episode > 0 and IfPrint:
            print "episode: ", episode
            print total_reward
        s = env.reset()
        IfEnd = False
        total_reward = 0
        grad_logpolicy = theta * 0
        while not IfEnd:
            # feature = env.phi(s)
            act_dist = policy(theta,s)
            a = choose_action(act_dist)
            s, r, IfEnd = env.next_step(s,a)
            total_reward += r
            grad_logpolicy += env.ep_grad_log_policy(theta,s,a,act_dist)
        total_reward += env.ini_money
        total_reward = np.log(total_reward)
        total_r.append(total_reward)
        control = np.mean(total_r)
        beta1 = max(beta / (episode + 1), 0.0001)
        alpha1 = max(alpha / (episode + 1) ** 0.6, 0.0001)
        tempep = 0
        temp1 = total_reward - control
        temp2 = Jhat * total_reward ** 2 - 2 * total_reward * Jhat ** 2 + tempep
        temp3 = 2 * Vhat + tempep
        theta += beta1 * (temp1 - temp2 / temp3) * grad_logpolicy / Vhat ** 2
        Jhat += alpha1 * (total_reward - Jhat)
        Vhat += alpha1 * (total_reward ** 2 - Jhat ** 2 - Vhat)
    # p.finish()
    savename = "./Portfolio_policy/Tamar/Tamar_theta_" + str(i) + ".txt"
    np.savetxt(savename,theta)
    if IfPrint:
        plt.figure(1)
        plt.plot(total_r,'g-',label = 'Tamar')
        plt.show()

    # value = Portfolio_run_once(theta)
    # print value

def American_Option_main(i):
    total_r = []
    s = env.reset()
    feature = env.phi(s)
    theta = feature * 0
    Jhat = 0.5
    Vhat = 1
    # p.start(episode_num)
    for episode in xrange(episode_num):
        # p.update(episode+1)
        # if episode % 10 == 0:
        if episode % 10 == 0 and episode > 0 and IfPrint:
            print "episode: ", episode
            print total_reward
        s = env.reset()
        IfEnd = False
        total_reward = 0
        grad_logpolicy = theta * 0
        while not IfEnd:
            # feature = env.phi(s)
            act_dist = policy(theta,s)
            a = choose_action(act_dist)
            s, r, IfEnd = env.next_step(s,a)
            grad_logpolicy += env.grad_log_policy(theta,s,a,act_dist)
            total_reward += r
        total_r.append(total_reward)
        control = np.mean(total_r)
        beta1 = beta #/ (episode + 1)
        alpha1 = alpha #/ (episode + 1) ** 0.6
        tempep = 1e-8
        temp1 = total_reward - control
        temp2 = Jhat * total_reward ** 2 - 2 * total_reward * Jhat ** 2 + tempep
        temp3 = 2 * Vhat + tempep
        theta += beta1 * (temp1 - temp2 / temp3) * grad_logpolicy / Vhat ** 2
        Jhat1 = Jhat +  alpha1 * (total_reward - Jhat)
        Vhat1 = Vhat + alpha1 * (total_reward ** 2 - Jhat ** 2 - Vhat)
        Jhat = Jhat1 + 0.0
        Vhat = Vhat1 + 0.0
    # p.finish()
    savename = "./" + environment + "_policy/Tamar/Tamar_theta_" + str(i) + ".txt"
    np.savetxt(savename,theta)
    if IfPrint:
        plt.figure(1)
        plt.plot(total_r,'g-',label = 'Tamar')
        plt.show()

if environment == "Portfolio":
    main = Portfolio_main
if environment == "American_Option" or environment == "Optimal_Stopping":
    main = American_Option_main

if __name__ == '__main__':
    for i in xrange(trails_num):
        print "trail#: ",i
        main(i)
